@@ -1,7 +1,7 @@
 #
 # This file is part of MooX-Options
 #
-# This software is copyright (c) 2011 by celogeek <me@celogeek.com>.
+# This software is copyright (c) 2013 by celogeek <me@celogeek.com>.
 #
 # This is free software; you can redistribute it and/or modify it under
 # the same terms as the Perl 5 programming language system itself.
@@ -12,11 +12,12 @@ package MooX::Options::Descriptive::Usage;
 
 use strict;
 use warnings;
-our $VERSION = '3.99';    # VERSION
+our $VERSION = '4.000';    # VERSION
 use feature 'say';
 use Text::WrapI18N;
 use Term::Size::Any qw/chars/;
 use Getopt::Long::Descriptive;
+use Scalar::Util qw/blessed/;
 
 my %format_doc = (
     's'  => 'String',
@@ -53,7 +54,10 @@ sub leader_text { return shift->{leader_text} }
 
 sub sub_commands_text {
     my ($self) = @_;
-    my $sub_commands = $self->{sub_commands} // [];
+    my $sub_commands
+        = defined $self->{target}
+        ? $self->{target}->_options_sub_commands() // []
+        : [];
     return if !@$sub_commands;
     return "", 'SUB COMMANDS AVAILABLE: ' . join( ', ', @$sub_commands ), "";
 }
@@ -77,14 +81,17 @@ sub _set_column_size {
 
 sub option_text {
     my ($self) = @_;
-    my $options = $self->{options};
-
+    my %options_data
+        = defined $self->{target} ? $self->{target}->_options_data : ();
+    my $getopt_options = $self->{options};
     my @message;
     _set_column_size;
-    for my $opt (@$options) {
+    for my $opt (@$getopt_options) {
         my ( $short, $format ) = $opt->{spec} =~ /(?:\|(\w))?(?:=(.*?))?$/x;
         my $format_doc_str;
         $format_doc_str = $format_doc{$format} if defined $format;
+        $format_doc_str = 'JSON'
+            if defined $options_data{ $opt->{name} }{json};
         push @message,
               ( defined $short ? "-" . $short . " " : "" ) . "-"
             . ( length( $opt->{name} ) > 1 ? "-" : "" )
@@ -98,15 +105,20 @@ sub option_text {
 }
 
 sub option_pod {
-    my ( $self, $options ) = @_;
+    my ($self) = @_;
 
-    my %options_config = $options->_options_config;
-    my %options_data   = $options->_options_data;
+    my %options_data
+        = defined $self->{target} ? $self->{target}->_options_data : ();
+    my %options_config
+        = defined $self->{target} ? $self->{target}->_options_config : ();
 
     my $prog_name = $self->{prog_name}
         // Getopt::Long::Descriptive::prog_name;
 
-    my $sub_commands = $self->{sub_commands} // [];
+    my $sub_commands
+        = defined $self->{target}
+        ? $self->{target}->_options_sub_commands() // []
+        : [];
 
     my @man = ( "=head1 NAME", $prog_name, );
 
@@ -128,6 +140,8 @@ sub option_pod {
         my ( $short, $format ) = $opt->{spec} =~ /(?:\|(\w))?(?:=(.*?))?$/x;
         my $format_doc_str;
         $format_doc_str = $format_long_doc{$format} if defined $format;
+        $format_doc_str = 'JSON'
+            if defined $options_data{ $opt->{name} }{json};
 
         my $opt_long_name
             = "-" . ( length( $opt->{name} ) > 1 ? "-" : "" ) . $opt->{name};
@@ -173,7 +187,11 @@ sub option_pod {
 
 sub warn { return CORE::warn shift->text }
 
-sub die { return CORE::die shift->text }
+sub die {
+    my ($self) = @_;
+    $self->{should_die} = 1;
+    return;
+}
 
 use overload (
     q{""} => "text",
@@ -196,7 +214,7 @@ MooX::Options::Descriptive::Usage - Usage class
 
 =head1 VERSION
 
-version 3.99
+version 4.000
 
 =head1 DESCRIPTION
 
@@ -255,7 +273,7 @@ Croak your options help message
 =head1 BUGS
 
 Please report any bugs or feature requests on the bugtracker website
-https://tasks.celogeek.com/projects/perl-modules-moox-options
+https://github.com/celogeek/MooX-Options/issues
 
 When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
@@ -267,7 +285,7 @@ celogeek <me@celogeek.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by celogeek <me@celogeek.com>.
+This software is copyright (c) 2013 by celogeek <me@celogeek.com>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
